#ifndef FRIEDER_NAKE_H
#define	FRIEDER_NAKE_H

/* 'Code'/comments written by Frieder Nake for Polytik

To start with, I must warn: the following is a comment only from a meeting between John Richards and Frieder Nake in Bremen. It is a comment written to John the day after he had left.

When you start developing a program, you must always first write a comment. That comment should go into the first part of the code. The reason for this is that this comment is essentially the specification of what the program will have to do.  And, without any concept, you would not write a program, would you?

What in art (and elsewhere) is the concept, in conceptual art already becomes the final work (this is so only in an ideal situation).

Like Yoko Ono and so many others, we should write postcards to each other. Each postcard would transport one concept. And that concept is the work. Like these:

Go into the forest, embrace a tree, sing a song, and leave again.

Or set up an electronic circuit on a yellow board, upload a program in C to a blue board and run it and fiddle around with the knobs and switches.

Are the two not the same, if you see them from some distant point of view? 
 */

/*
Discussion with Frieder Nake, Bremen, 29 September 2016
Setting up Polytik in a room at the University of the Arts (HfK), Dechanatstrasse, Bremen

JR
Still enough stuff you see.

FN
Yes, but small. I mean, when you brought out these little boxes ... well they're tiny almost, so very efficient. Amazing!

JR
In many ways, I'm hugely influenced by the culture of mobility.

FN
Oh! A different word. Are you a digital nomad?

JR
Ah, well I could be ... I was more of a neo-Luddite ... a kind of digital Ludditism. But that's not true actually. Some people think my work is a reaction to a lot of digital technology. It's true only on a superficial level. They don't see how digital concepts have informed the thinking and the behaviours. You could argue some of this is also informed by mobile practice and cultures, which is hugely informed by new technologies, size, and the miniaturisation of technology; or is it more about being self-sufficient in a technological milieu? The recording studios dominated some of the early forms of electronic music. That's a very constraining species, and to make electronic music outside of the studio is an incredibly liberating prospect. So part of this [Polytik] is about that. It's also about access. You could make electronic music on, for example, a mobile phone, but it's a different type of engagement there. You've talked about some of this as well ... how certain types of technology are driven by the market or war. At least here [Polytik] I've got some kind of ownership of what I'm trying to do with the technology. Of course these components still could exist in some terrible weapon or in another context, but I can somehow reshape this stuff in a different way, or use this technology for my ends rather than be dominated by it. I'm not naive enough to think that by making or designing it myself that I'm completely in control or free from being a slave to this type of technology, but it gives me some kind of independence. The fact is, I'm choosing the parts of the technology I'm going to use. I think this is quite critical for people working in new media or with technology. And it's a kind of reductionism rather than me just making music with this [laptop]. If I could limit my boundaries, then maybe that would give me a chance of being rigorous ... Of course this can happen on here [laptop] as well but there are so many more distractions aren't there?

FN
I was thinking all the time, what, the last fifteen minutes or so, about this encounter here of your own designs produced by yourself, and then this machine [laptop] next to it, which, of course just looking at size, is tremendously easy to carry and contains a lot, lot, lot of memory and computing power and functionalities. But of course lacks, totally lacks this self-sufficient side of what is happening to us as human beings in this time. When we think of only fifty years ago, the electronic music studio and the computing centre were totally dominating. We had huge spaces full of electronics that nobody had access to. And that has changed. All this is possible for you to do because of that great development. The miniaturisation of [technology] all the time further and further, seemingly without an end.

JR
It's too small. You can't pick it up! Already, this [Polytik] is old electronics.

FN
Which means that dialectically looking at things, there are no movements in the world under our historic perspective that continue and continue and continue. There will either by sheer size, or sheer lack of size. There are ends. And are they [these ends] metaphysically coming about?  No, no, no! They come about because I believe there is some resistance in the human being despite everything we learn about how we will be supplanted and become part of computing equipment. Maybe a dictator will rise up somewhere. Yes, of course he could do this [supplant us by technology] for a while, but the human being, as a species, is slow and smart enough to set certain boundaries to what is happening.

JR 
A hunter-gatherer, always. People often say, 'I like this.' [Dirty Electronics Cardboard Box Amp]. In my life, my career, the best things I've done musically have often been to take a stance or have some kind of resistance against going in a certain direction. I've been surrounded by very expensive loudspeakers and wonderful equipment, but a resistance to using it has been very healthy. And now I have to be careful in that I need to also create a resistance against ideas of objecthood and all the materialism that that [objecthood] courts. These [Polytik modules] can be exploited as objects; so at this stage, having created by accident a product, I have to ensure I have some kind of resistance against that. This is the dilemma I find myself in. Interestingly you suggested that there were similar things that happened in your life when you started exhibiting works, and how it was often the curator who decided how the art would be presented. You created a resistance against that and stopped exhibiting for a while. ... Because you're interested in algorithms, there is possibly no definitive version of the artwork anyway. You create the program and then you run a version of it. How do you feel about that? How do you feel about the material nature of some of your work?

FN
That is, I believe, this question of what it contains. Maybe what I'm going to say is too heavy, but as a starting point, a point from where to reflect, I think it's valid, namely, the question of, 'How is the individual work, in whichever way it is being produced by an algorithm, being executed?'; so I'm dealing with an algorithm and the individual work. This is at the heart of everything we are doing. 'We are doing' means you concentrating on sounds, somebody else on moving images, someone else on still images, and continuing the algorithm ... the algorithm 'keeping' running. That's at the heart of this. In my life as just a young guy, this was not immediately there as an important question in an historic dimension. In the beginning it was just 'I'm doing something here and I'm pretty sure nobody else is doing this.' ... And what you are doing here is amazing because it's mis-abusing that machinery.

My job as a student in the computing centre at the University of Stuttgart was to look after what was called the program library, something that nobody [now] believes ever existed. In this case, the output and input medium was paper tapes not punch cards, but that's not a real difference: it just took up less space. There was one of these metal trunks that contained the subprograms so to speak. That was the term then. A subprogram routine was something that the engineer, who now came from their [Zuse] department, wanted us in the computing centre to set up so that he could solve these differential equations or whatever: some statistics maybe, some static calculations, whatever it was. Our job was for more general, to provide programs that would then function as subprograms to the main program that he, by the way, didn't write himself. He needed to hire somebody here to do it [write the subprograms]. And that person [engineer] would never finish the job unless there were those subprograms, to solve for instance a system of numeric equations, find out elements and so forth. And this was my job as a student in fourth, sixth semester. Fantastic! I knew I was doing something here independent in academia so to speak.

And then one day, John, this little story must be told, one day in the morning our professor - the boss of the centre was young and friendly, really friendly, from Vienna, so they're a little slower -  he came to tell me, 'You know what,' in his beautiful accent, 'we are buying a drawing machine.' So I said, 'A ha,' not really knowing what he was talking about. I knew what drawing is, what a machine is, now there coming with a machine, a drawing machine!  So he continues by saying, 'But we will not get any software.' This much I knew, the kind of machinery we have here in this computing centre always required to have some software otherwise it would not do anything. 'Ah, no software,' and after a short dramatic pause, he then asked, 'Would you do it?' Fantastic question! This short, minute long, moment changed my life. I only much later realised how great this professor was because he put a question forward to a student of whom he had no idea would be able to do what he was asking; so he trusted me. The category of trust has later become my major, focal point for teaching. I'm opposed to all exams because they are an expression of mistrust, 'We don't trust you guys that you do anything or that you've learnt it.'

Ok, but that's an aside. I'm also proud of my reaction to his question, because my answer was, 'Yes!' I didn't ask him are there any books, or what do I have to know, or maybe I go home and study something. I said yes. No maybes, no nothing!  Not having any idea of what that meant. If I now say yes, I have to write the software for a machine I've never seen, I have no idea of what it is doing, and I [have to] develop a package that you would nowadays call a basic drawing package, a Calcom package so to speak. Without seeing the machine! I only got the specifications of that machine from the company.  The specifications were that machine expected some codes. It expected codes of a very simple kind. It was a machine that looking down upon it was rather large, one point five metres squared. There was something along here and here [scribbles in notebook], and this was a spindle like this [scribbles in notebook] along which this switch could be moved, and inside here also such a thing along which the drawing head, as it was called, could move; so the two, of course, together give you all movements, not quite, because it was digital, not analogue.

JR
Hard to do curves then?

FN 
Yes, but [it did curves] quite well. The resolution, that would be the word nowadays, was one sixteenth of a millimetre; so for the time, for everything mechanical, that was quite good. If this is a sixteenth of millimetre in width, then you could go something millimetre in all directions. But if you had something in between, then you had to go in this case, like this, maybe like this [scribbles in notebook] and then take that most likely [route]. I learned, and this I had to develop myself, what everybody nowadays - and nowadays means since many, many years - calls Bresenham analysis [a way of plotting a line between two points]. This anecdote I've told you happened in 1963. When he [my professor] asked me to do this.

I must tell you, in order to show the stupidity of such a young student, I was extremely proud doing this and I had now prepared some paper tapes, still not having the machine, but they told us come to the place where they [kept the machine]. The machine was actually the last invention of Konrad Zuse that German computer guy; so I was invited to come to their place, to the Zuse factory, which made me extremely proud. But I was outrageously proud, mainly believing these paper tapes I had produced, when I went there [to the factory], would immediately work. I took a train, an old train. On the train I was reading, I still know this, Wittgenstein's Tractatus. Would you believe it! There was an old couple sitting there, I was sitting here [pointing]. I held it up so that they could read what I was reading [laugh], not assuming they would understand, but I hoped they would think, 'Oh this guy must be extremely bright. What is he reading?' It was amazing that in that moment when I was about to get, for a short moment only, into a situation that could change my life, I was reading this great philosopher. Fantastic! I loved that. I arrived there in the evening. They paid for a hotel for me. Had I ever been to a hotel? Most likely not, and the next morning I went to the factory. These engineers said, 'Hello!' 'I want to test this now,' [I replied]. 'What is it?' [they asked]. [I said], 'We are supposed to have produced this software for that huge machine because you're not doing this.' 'OK come over here,' [called the engineers]. So I put the paper tape into the machine. They started it. It started to move, it put the pen down, it did a little thing, and then it got stuck. The engineers, I believe, sensed that this guy [myself] was way too proud. I was proud, and was of course the opposite now, totally disappointed. Totally disappointed! It was possible for me to find out [the problem]. I sat down, fiddled around to correct it and we got it working. The lesson I learned here, and this is in 1964, is - and in Germany you didn't have computer science as a discipline, which came five years later - be modest with programs. You are always doing things wrong. And the testing is what you have to learn. How to test it. What I had done before of course was I calculated the program and run through the program by hand, which is a good exercise. Nobody would do it nowadays, nobody. It would be totally crazy. The lessons I learned there in all their slow development was such that I believe they are still important for me nowadays, in terms of modest behaviour is probably what we should always have.

JR
What's interesting there perhaps is that you started that programming separate from the machine, purely from a conceptual point of view. This is in many ways what I've asked you through email. Could we come up with a pattern or conceptualise some kind of code that could run on here [Polytik]? I'm taking you back to [your experiences of] 1963 where it also becomes incredibly important aside from the materials. It will be interesting to see how far we get with that and whether we create short code that has a character separate from these objects that realise it. These objects [Polytik] simply just realise this algorithm, this simple array, but there still may be some character in that. That still remains a dilemma for me, partly that you work on this code, and we test it in this domain, and then we go back and forth in it. At what point does it become finished? Where's the art creation? Where's the focus in many ways, or is it some kind of weird hybrid, particularly the way in which this code will become abstracted through the analogue world. But it's interesting that you came to that purely conceptually.

FN
I do believe in concepts. I still do believe in them. At the moment when the machine stops, just after having started, so to speak, telling me, 'Look, yeah ok. I read you. Nice that you're here. However, sorry I must tell you I'm stuck. You are too stupid.' That moment is also important.  Until then it is conceptualisation. And by believing in the power of this [conceptualisation], I want to stay in command. I have gone through waves of this issue, the relationship between human and machine, and I feel as if I have a total security moving through here. I'm never in any doubt at all, almost all my students are, but I go through like a Christian or something like this.

JR
You have faith. In the sense it's about your relationship with it [the machine] and your idea of control and non-control.

FN
But on the other hand, that's the dialectics of this. We are surrounded by material conditions to use a more general term, material conditions, particularly in the form of a machine, and those material conditions we tend to believe we can get beyond by conceptualising something.  That is wrong. Our world has largely become controlled by, not really but to a large extent, by signs, by semiotic processes away from 'thing' processes. Now the nice thing about signs is they never appear without some materiality. There are no signs in the world. There is only matter in the world and it takes on forms that we then take as a sign of whatever it is.  The thunder is a sign of, whatever.
*/
#endif	/* FRIEDER_NAKE_H */

