// Polytik_v1_3_white 30/10/17
// John Richards (Dirty Electronics)
// Code for POLYTIK Core (White) October 2017 - distributed by bleep.com
// External sync set to trigger +5v
// Glyphs 0 - 27: Generic patterns (Dirty Electronics)
// Glyph 28: Jas Shaw (polyrhythmic pattern)
// Glyph 29: Mark Fell (discussion with John Richards as comments)
// Glyph 30: Frieder Nake (discussion with John Richards as comments)
// Glyph 31 (dot): User-input (programmable)
// pic18f26k22
// MPLAB X IDE v3.55
// xc8-v1.41 (compiler)
// Acknowledgments: thanks to Jim Frize

#include <htc.h>
#include <stdlib.h>
#include <pic18f26k22.h>
#include "config.h"
#include "PWM_values.h"
#include "display.h"
// Guest artist code
#include "Mark_Fell.h"
#include "Frieder_Nake.h"


#define _XTAL_FREQ 16000000
#define SWITCH PORTAbits.RA1 // counter tactile switch up
#define SWITCH2 PORTAbits.RA2 // counter tactile switch down
#define SWITCH3 PORTAbits.RA5 // user sequence input switch (RA5 = pin 7)
#define SWITCH4 PORTAbits.RA6 // user sequence input 2 switch (RA6 = pin 10)
#define TRIGGER PORTAbits.RA7 // CV sequence trigger - index 0

// Implements a delay of a number of milliseconds.
void delay(int ms) {
    while (ms-- > 0) {
        __delay_ms(1);
    }
}

//GLOBAL VALUE
int inc = 0; // increment value - in ISR
int dec = 0; // decrement value - in ISR
int count = 0; // counter for timer in ISR

unsigned char ctr = 1; // counter for presets
unsigned char ctr2 = 1; // counter variable user input

// User input arrays
char user_input[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // user generated sequence
char current_index = 0;
char array_length = 0; // index for user_input array

char user_input2[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // user generated sequence
char current_index2 = 0;
char array_length2 = 0; // index for user_input array

// PWM counter
char value1 = 0;
char value2 = 0;
char value3 = 0;
char value4 = 0;
char value5 = 0;
char value6 = 0;
char value7 = 0;
char value8 = 0;
char value11 = 0;
char value12 = 0;
char value16 = 0;
char value20 = 0;
char value32 = 0;
char value50 = 0;
char value64 = 0;
char value_r4 = 0;
char value_r8 = 0;
char value_r16 = 0;
char value_r32 = 0;
char value_r255 = 0;


// Initialises the variable resistor ADC configuration. Variable resistor for tempo/speed of sequence.
void initADC() {
    // 1. Configure Port:
    //  - Disable pin output driver(See TRIS register)
    TRISAbits.RA0 = 1;

    //  - Configure pin as analog
    ANSELAbits.ANSA0 = 1;

    // 2. Configure the ADC module:
    //  - Select ADC conversion clock
    ADCON2bits.ADCS = 0b101; // Fosc/16

    //  - Configure voltage reference
    ADCON1bits.PVCFG = 0; // +ref is VDD
    ADCON1bits.NVCFG = 0; // -ref is VSS

    //  - Select ADC input channel
    ADCON0bits.CHS = 0; // connect AN0 to the ADC

    //  - Select result format
    ADCON2bits.ADFM = 0; // left justified

    //  - Select acquisition delay
    ADCON2bits.ACQT = 0b101; // 12 TAD

    //  - Turn on ADC module
    ADCON0bits.ADON = 1;
}


// Reads the value of the variable resistors into a single byte.
unsigned int readADC(unsigned char channel) {
    unsigned int AN_Val;

    // Select desired channel for ADC operation
    switch (channel) {
        case 1: // AN0
            ADCON0bits.CHS = 0;
            break;
        case 2: // AN3
            ADCON0bits.CHS = 3;
            break;
        default: // Any other value will default to AN0
            ADCON0bits.CHS = 0; // AN0
    } // switch (channel)

    // Channel selected proceed with ADC conversion
    __delay_us(10); // sampling time
    // start a conversion
    ADCON0bits.GO = 1; //This sets the go/!done bit that starts conversion. Bit will be cleared when ADC is complete

    while (ADCON0bits.GO == 1)
        __delay_us(10); // longer delay here influences the ISR and timers

    AN_Val = ADRESH; //returns 8-bit?

    return AN_Val; // int ADC(unsigned char channel)
}

//Initialises PWM on the three PWM generators.
void initPWM() {
    // See PIC18F26K22 data sheet p. 186
    // 1. Disable the CCPx pin output driver by setting the associated TRIS bit.
    TRISC = 0b00000110; // RC1 (PWM2) and RC2 (PWM1)
    TRISBbits.RB5 = 1; //PWM3 pin 26
    TRISBbits.RB0 = 1; //PWM4 pin 21
    TRISAbits.RA4 = 1; //PWM5 pin 6

    // 2. Select the 8-bit TimerX resource, (Timer2, Timer4 or Timer6) to be
    //    used for PWM generation by setting the CxTSEL bits in the
    //    CCPTMRSx register.
    CCPTMRS0bits.C1TSEL = 0; // user timer 2

    // 3. Load the PRx register for the selected TimerX with the PWM period value.
    PR2 = 0x40; // decimal 64 period

    // 4. Configure the CCP module for the PWM mode by loading the CCPxCON
    //    register with the appropriate values.
    CCP1CONbits.CCP1M = 0b1100;
    CCP2CONbits.CCP2M = 0b1100;
    CCP3CONbits.CCP3M = 0b1100;
    CCP4CONbits.CCP4M = 0b1100;
    CCP5CONbits.CCP5M = 0b1100;

    // 5. Load the CCPRxL register and the DCxB bits of the CCPxCON register,
    //    with the PWM duty cycle value.
    CCPR1L = 128 >> 2;
    CCPR2L = 128 >> 2;
    CCPR3L = 128 >> 2;
    CCPR4L = 128 >> 2;
    CCPR5L = 128 >> 2;

    CCP1CONbits.DC1B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP2CONbits.DC2B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP3CONbits.DC3B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP4CONbits.DC4B = 128 & 0b00000011; // 128 is 50% duty cycle
    CCP5CONbits.DC5B = 128 & 0b00000011; // 128 is 50% duty cycle

    // 6. Configure and start the 8-bit TimerX resource:
    //    - Clear the TMRxIF interrupt flag bit of the
    //      PIR2 or PIR4 register. See Note 1 below.
    PIR1bits.TMR2IF = 0;

    //    - Configure the TxCKPS bits of the TxCON
    //      register with the Timer prescale value.
    T2CONbits.T2CKPS = 0b01; // prescale 1:4 p. 177. This is not 8(0b11)!

    // - Enable the Timer by setting the TMRxON
    // bit of the TxCON register.
    T2CONbits.TMR2ON = 1;

    // 7. Enable PWM output pin:
    //    - Wait until the Timer overflows and the TMRxIF bit of the PIR2 or PIR4
    //      register is set. See Note 1 below.
    while (PIR1bits.TMR2IF == 0) {
    }

    //    - Enable the CCPx pin output driver by clearing the associated TRIS bit.
    TRISC = 0; // RC1 (PWM2) and RC2 (PWM1)
    TRISBbits.RB5 = 0; //PWM3
    TRISBbits.RB0 = 0; //PWM4
    TRISAbits.RA4 = 0; //PWM5
}


//Sets the PWM values for each of the PWM controllers.
void setPWM(char value1, char value2, char value3, char value4, char value5) {
    CCPR1L = value1 >> 2;
    CCP1CONbits.DC1B = value1 & 0b00000011;

    CCPR2L = value2 >> 2;
    CCP2CONbits.DC2B = value2 & 0b00000011;

    CCPR3L = value3 >> 2;
    CCP3CONbits.DC3B = value3 & 0b00000011;

    CCPR4L = value4 >> 2;
    CCP4CONbits.DC4B = value4 & 0b00000011;

    CCPR5L = value5 >> 2;
    CCP5CONbits.DC5B = value5 & 0b00000011;
}


// interrupt service routine
void interrupt ISR(void) {
      
    if (ctr2 == 1)
        user_input[array_length] = readADC(2); // read_analogue_value channel (case) 2

    else
        if (ctr2 == 2)
        user_input2[array_length2] = readADC(2); // read_analogue_value channel (case) 2
   
      int r4 = rand() % 4;
      int r8 = rand() % 8;
      int r16 = rand() % 16;
      int r32 = rand() % 32;
      int r255 = rand() % 255;
    
    // has the interrupt been fired by timer0?
    if (TMR0IF == 1) {
        count++;

        if (count >= (255 / (readADC(1) + 1) * 2))
        {
            
            //interrupt has been fired
            //forwards, backwards and random parameters for PWM arrays - see play func
            inc++; //forward
            dec--; //backward
            current_index++; // user_input1 forward
            current_index2++; // user_input1 backward
            
            value1 = inc % 1;
            value2 = inc % 2;
            value3 = inc % 3;
            value4 = inc % 4;
            value5 = inc % 5;
            value6 = inc % 6;
            value7 = inc % 7;
            value8 = inc % 8;
            value11 = inc % 11;
            value12 = inc % 12;
            value16 = inc % 16;
            value20= inc % 20;
            value32 = inc % 32;
            value50 = inc % 50;
            value64 = inc % 64;
            value_r4 = r4; // random 0 - 4
            value_r8 = r8; // random 0 - 8
            value_r16 = r16; // random 0 - 16
            value_r32 = r32; // random 0 - 16
            value_r255 = r255; // random 0 - 16

            // reset rand if triggered
            if (TRIGGER == 1)
            r4 = 0;
            r8 = 0;
            r16 = 0;
            r32 = 0;
            r255 = 0;

            // reset counter
            count = 0;
        }
        // reset the overflow interrupt flag
        TMR0IF = 0;
    }
}

void init_timer0(void) {

    T0CON = 0x00; //00000000 - stop timer; 16-bit; internal; source edge; pre-scaler select; pre-scale values
    //Timer will run from Internal instruction cycle clock. Pre-scaler will be used

    T0CONbits.T08BIT = 1; //USING Timer0 as 16-bit counter 1 = 8-bit; 0 = 16-bit
    //T0CONbits.PSA = 0; //set to pre-scaler
    T0CONbits.T0PS = 0b110; // pre-scale
    T0CONbits.TMR0ON = 1; //Turned ON timer0

    INTCONbits.TMR0IE = 1; //Enabling Timer0 Interrupt

}

void play() {

PORTC = ~DisplayC(ctr);
PORTB = ~DisplayB(ctr);

    if (ctr == 1) {
        setPWM(PWM_A_1[value8],
                PWM_B_1[value8],
                PWM_C_1[value8],
                PWM_D_1[value8],
                PWM_E_2[value8]);
    } else if (ctr == 2) {
        setPWM(PWM_A_2[value8],
                PWM_B_2[value_r8],
                PWM_C_2[value8],
                PWM_D_2[value_r8],
                PWM_E_2[value8]);
    } else if (ctr == 3) {
        setPWM(PWM_A_3[value_r8],
                PWM_B_3[value8],
                PWM_C_3[value8],
                PWM_D_3[value8],
                PWM_E_2[value8]);
    } else if (ctr == 4) {
        setPWM(PWM_A_4[value_r8],
                PWM_B_4[value8],
                PWM_C_4[value_r8],
                PWM_D_4[value_r8],
                PWM_E_3[value8]);
    } else if (ctr == 5) {
        setPWM(PWM_A_5[value8],
                PWM_B_5[value8],
                PWM_C_5[value8],
                PWM_D_4[value_r8],
                PWM_E_1[value8]);
    } else if (ctr == 6) {
        setPWM(PWM_A_6[value8],
                PWM_B_6[value_r8],
                PWM_C_6[value8],
                PWM_D_4[value_r8],
                PWM_E_1[value8]);
    } else if (ctr == 7) {
        setPWM(PWM_A_7[value8],
                PWM_B_7[value_r8],
                PWM_C_7[value8],
                PWM_D_4[value_r8],
                PWM_E_1[value_r8]);
    } else if (ctr == 8) {
        setPWM(PWM_A_8[value_r8],
                PWM_B_8[value_r8],
                PWM_C_8[value_r8],
                PWM_D_2[value8],
                PWM_E_4[value8]);
    } else if (ctr == 9) {
        setPWM(PWM_A_9[value_r8],
                PWM_B_9[value_r8],
                PWM_C_9[value8],
                PWM_D_4[value8],
                PWM_E_3[value8]);    
    } else if (ctr == 10) {
        setPWM(PWM_A_10[value8],
                PWM_B_10[value8],
                PWM_C_10[value8],
                PWM_D_4[value_r8],
                PWM_E_1[value8]);
// Thanks to Petros Galanakis 11 - 14        
    } else if (ctr == 11) {
        setPWM(PWM_A_11[value16],
                PWM_B_11[value16],
                PWM_C_11[value8],
                PWM_D_11[value16],
                PWM_E_1[value8]);
    } else if (ctr == 12) {
        setPWM(PWM_A_12[value12],
                PWM_B_12[value8],
                PWM_C_12[value8],
                PWM_D_12[value12],
                PWM_E_12[value_r16]);
    } else if (ctr == 13) {
        setPWM(PWM_A_13[value8],
                PWM_B_13[value8],
                PWM_C_13[value8],
                PWM_D_13[value8],
                PWM_E_13[value4]);
    } else if (ctr == 14) {
        setPWM(PWM_A_14[value32],
                PWM_B_14[value8],
                PWM_C_14[value8],
                PWM_D_14[value32],
                PWM_E_14[value12]);
// Some arpeggios for Combi and VCO
    } else if (ctr == 15) {
        setPWM(PWM_A_15[value16],
                PWM_B_15[value16],
                PWM_C_15[value16],
                PWM_D_15[value3],
                PWM_E_4[value8]);    
    } else if (ctr == 16) {
        setPWM(PWM_A_16[value12],
                PWM_B_16[value8],
                PWM_C_16[value4],
                PWM_D_16[value4],
                PWM_E_16[value16]);
    } else if (ctr == 17) {
        setPWM(PWM_A_17[value32],
                PWM_B_17[value32], // doubled
                PWM_C_17[value8],
                PWM_D_17[value8],
                PWM_E_17[value8]);
    } else if (ctr == 18) {
        setPWM(PWM_A_18[value16],
                PWM_B_18[value16],
                PWM_C_18[value8],
                PWM_D_18[value8],
                PWM_E_18[value4]);
    } else if (ctr == 19) {
        setPWM(PWM_A_19[value32],
                PWM_B_19[value16],
                PWM_C_19[value12],
                PWM_D_19[value6],
                PWM_E_19[value32]);
// Some noise
    } else if (ctr == 20) {
        setPWM(PWM_A_20[value_r255],
                PWM_B_20[value_r255],
                PWM_C_20[value_r255],
                PWM_D_20[value4],
                PWM_E_20[value_r255]);
// John R extended - poly rhythms against PWM_B/C/D etc.       
    } else if (ctr == 21) {
        setPWM(PWM_A_21[value8],
                PWM_B_21[value8],
                PWM_C_21[value32],
                PWM_D_21[value8],
                PWM_E_21[value16]);
    } else if (ctr == 22) {
        setPWM(PWM_A_22[value5],
                PWM_B_22[value8],
                PWM_C_22[value3],
                PWM_D_22[value3],
                PWM_E_22[value12]);        
    } else if (ctr == 23) {
        setPWM(PWM_A_23[value7],
                PWM_B_23[value8],
                PWM_C_23[value4],
                PWM_D_23[value8],
                PWM_E_1[value8]);
// Change ringing patterns for Combi and VCO
    // Jokers
    } else if (ctr == 24) {
        setPWM(PWM_A_24[value11],
                PWM_B_24[value8],
                PWM_C_24[value8],
                PWM_D_24[value8],
                PWM_E_24[value11]);
    // Kings        
    } else if (ctr == 25) {
        setPWM(PWM_A_25[value11],
                PWM_B_25[value_r4],
                PWM_C_25[value3],
                PWM_D_25[value2],
                PWM_E_1[value8]); // static value
    //Tittums        
    } else if (ctr == 26) {
        setPWM(PWM_A_26[value11],
                PWM_B_26[value8],
                PWM_C_26[value8],
                PWM_D_26[value8],
                PWM_E_6[value8]);
    //Plain Hunt       
    } else if (ctr == 27) {
        setPWM(PWM_A_27[value50],
                PWM_B_27[value_r8], // first 5 values and loops
                PWM_C_27[value8],
                PWM_D_27[value6],
                PWM_E_4[value8]);   
// Guest artists
    // Jas Shaw
    } else if (ctr == 28) {
        setPWM(PWM_A_28[value7],
                PWM_B_28[value4],
                PWM_C_28[value7],
                PWM_D_28[value7],
                PWM_E_28[value8]);
    // Mark Fell - see Mark_Fell.h
    } else if (ctr == 29) {
        setPWM(PWM_A_29[value1],
                PWM_B_29[value1],
                PWM_C_29[value1],
                PWM_D_29[value1],
                PWM_E_29[value1]);
    //Frieder Nake - see Frieder_Nake.h
    } else if (ctr == 30) {
        setPWM(PWM_A_30[value1],
                PWM_B_30[value1],
                PWM_C_30[value1],
                PWM_D_30[value1],
                PWM_E_30[value1]);  
// user-input (dot)
    } else if (ctr == 31) {
        setPWM(user_input[current_index],
                user_input2[current_index2],
                PWM_C_30[value1],
                PWM_D_30[value1],
                PWM_E_30[value1]);
    }
}

void main() {
    // set up the internal oscillator
    OSCCONbits.IRCF = 0b111; // 16MHz

    // up down tactile switches
    TRISAbits.RA1 = 1; // RA1 pin 3 as input SW1 (up) - 0 output/1 input
    TRISAbits.RA2 = 1; // RA2 pin 4 as input SW2 (down)
    TRISAbits.RA5 = 1; // RA5 pin 7 user sequencer input
    TRISAbits.RA6 = 1; // RA5 pin 10 user sequencer input 2

    // CV trigger input
    TRISAbits.RA7 = 1; // RA7 pin 9 as trigger input for reset table
    //TRISBbits.RB1 = 1; //test delete - input

    ANSELAbits.ANSA1 = 0; //  - Configure pin as analog 1 digital 0
    ANSELAbits.ANSA2 = 0; //  - Configure pin as analog 1 digital 0
    ANSELAbits.ANSA5 = 0; //  - Configure pin as analog 1 digital 0

    TRISC = 0;
    TRISB = 0; //needs to be selected

    // interrupt settings
    INTCONbits.GIE = 1; // Global Interrupt Enable
    
    // init the PWM
    initPWM();

    // init the ADC
    initADC();

    // initialize timer
    init_timer0();

    while (1) {

        // pre-set counters
        if (ctr == (31 + 1)) // if counter reaches 21, 1 more than presets,
            ctr = 1; // reset counter (min no.)
        else
            if (ctr == 0) // if counter reaches 0
            ctr = 31; // reset counter (max no.)

        // toggle between user_input 1 and 2
        if (ctr2 == (2 + 1)) // if counter reaches 3, 1 more than presets,
            ctr2 = 1; // reset counter (min no.)

        if (TRIGGER == 0) {
            play();
        }

        if (TRIGGER == 1) // if the gate/trigger is on
        {
            // reset the sequence

            inc = 0;
            dec = 0;
            current_index = 0;
            current_index2 = 0;
            //r = 0; // see ISR
            //r1 = 0;
            count = 0; // reset ISR counter
            value1 = 0;
            value2 = 0;
            value3 = 0;
            value4 = 0;
            value5 = 0;
            value6 = 0;
            value7 = 0;
            value8 = 0;
            value11 = 0;
            value12 = 0;
            value16 = 0;
            value20 = 0;
            value32 = 0;
            value50 = 0;
            value64 = 0;
            value_r4 = 0;
            value_r8 = 0;
            value_r16 = 0;
            value_r32 = 0;
            value_r255 = 0;
        }

        if (SWITCH == 0) {
            delay(50); // debounce
            if (SWITCH == 0) // if still pressed
                ctr++; // increment the counter

            // reset all values
            inc = 0;
            dec = 0;
            //r = 0;
            //r1 = 0;
            count = 0;
            value1 = 0;
            value2 = 0;
            value3 = 0;
            value4 = 0;
            value5 = 0;
            value6 = 0;
            value7 = 0;
            value8 = 0;
            value11 = 0;
            value12 = 0;
            value16 = 0;
            value20 = 0;
            value32 = 0;
            value50 = 0;
            value64 = 0;
            value_r4 = 0;
            value_r8 = 0;
            value_r16 = 0;
            value_r32 = 0;
            value_r255 = 0;
            delay(50); // debounce
        }

        if (SWITCH2 == 0) {
            delay(50); // debounce
            if (SWITCH2 == 0) // if still pressed
                ctr--; // decrement the counter if switch is pressed
            // reset all values
            inc = 0;
            dec = 0;
            //r = 0;
            //r1 = 0;
            count = 0;
            value1 = 0;
            value2 = 0;
            value3 = 0;
            value4 = 0;
            value5 = 0;
            value6 = 0;
            value7 = 0;
            value8 = 0;
            value11 = 0;
            value12 = 0;
            value16 = 0;
            value20 = 0;
            value32 = 0;
            value50 = 0;
            value64 = 0;
            value_r4 = 0;
            value_r8 = 0;
            value_r16 = 0;
            value_r32 = 0;
            value_r255 = 0;
            delay(50); // debounce
        }
            
        if (SWITCH3 == 0 && ctr2 == 1) {
            delay(40); // debounce .. holding SWITCH3 for user_input live
            if (SWITCH3 == 0 && ctr2 == 1)// if still pressed
                array_length++; // now increment to the next space in the array
            delay(40); // debounce
            if (array_length >= 16)
                array_length = 0; // if we reach the end of the array, go back to the start
        }

        if (SWITCH3 == 0 && ctr2 == 2) {
            delay(40); // debounce
            if (SWITCH3 == 0 && ctr2 == 2)// if still pressed
                array_length2++; // now increment to the next space in the array
            delay(40); // debounce
            if (array_length2 >= 16)
                array_length2 = 0; // if we reach the end of the array, go back to the start
        }

        if (SWITCH4 == 0) {
            delay(50); // debounce
            if (SWITCH4 == 0) // if still pressed
                ctr2++ == 0; // increment the counter
            delay(50); // debounce
        }

        if (current_index > array_length) {
            current_index = 0;
        }

        if (current_index2 > array_length2) {
            current_index2 = 0;
        }
    }
}